// Library client using the DII functionality to talk to
// server

#include <iostream.h>
#include <unistd.h>
#include <corbadii.h>

int main(int argc, char *argv[])
{
	if (argc < 3) {
		cout << "library_client: You must specify an author and title"
			<< endl;
		exit(1);
	}
	CORBA::Environment 	env;
	CORBA_DII::Request *req = CORBA_DII::Request::create("library", "add_book",
					"testrep", env);
	if ( env.check_exception() ) {
		cout << "Error creating request. " << endl;
		cout << "Exception: " << env.exception_value()->_id()
			<< " raised." << endl;
		return 0;
	}
	req->bind();
	CORBA::Environment& reqEnv = req->_environment();
	if ( reqEnv.check_exception()) {
		CORBA::Exception *exp = reqEnv.exception_value();
		CORBA::SystemException *system_exp = 
				CORBA::SystemException::_cast(exp);
		if (system_exp != NULL) {
			cout << exp->_id() << " occured: " << endl; 
			cout << "	with minor code " <<
				system_exp->minorFlag() << endl
				<< "	with completion status" <<
				system_exp->completed() << endl;
		}
		else {
			cout << "Exception: " << exp->_id() 
				<< " raised." << endl;
		}
		return(0);
	}
	else 
		cout << "Bound to library server" << endl;

	// Set the book values
	CORBA::Value *bookValue = req->arg("book_info");
	bookValue->member("author") = argv[1];
	bookValue->member("title") = argv[2];
	
	req->invoke();
	if (reqEnv.check_exception() ) {
		cout << "Exception: " << reqEnv.exception_value()->_id()
			<< " raised." << endl;
	}
	CORBA::Boolean ret = (CORBA::Boolean)(*req->result());
	if (ret == CORBA::TRUE)
		cout << "Added book " << argv[1] << ", " << argv[2] 
			<< " to list of books in library." << endl;
	else		
		cout << "Addition of book " << argv[1] << ", " << argv[2] 
			<< " to list failed." << endl;

	return(1);

}


