// Monitor client using DII
#include <signal.h>
#include <iostream.h>
#include <unistd.h>
#include <corbadii.h>


void sighandler(int, ...)
{
	exit(0);
}

int main(int argc, char *argv[])
{
	signal(SIGINT, sighandler);
        if ( argc < 3 ) {
                cout << "Usage: monclnt number1 number2" << endl;
                return 1;
        }
        CORBA::Environment env;
	// Create a request to communicate with Monitor
	CORBA_DII::Request *req = CORBA_DII::Request::create("Monitor", "get_server",
					"testrep", env);
	if ( env.check_exception() ) {
		cout << "Error creating request. " << endl;
		cout << env;
		return 0;
	}
	req->bind();
	CORBA::Environment& reqEnv = req->_environment();
	if ( reqEnv.check_exception()) {
		cout << "Unable to bind to Monitor" << endl;
		cout << reqEnv;
		return(0);
	}
	else 
		cout << "Bound to Monitor server" << endl;

	req->invoke();
	if (reqEnv.check_exception() ) {
		cout << "Exception: " << reqEnv.exception_value()->_id()
			<< " raised." << endl;
		return(0);
	}
	// Get the Compute Server Object
	CORBA::Object *computeServer = (CORBA::Object *)(*req->result());

	// Now create a new request to communicate with ComputeServer
	CORBA_DII::Request *computeReq = CORBA_DII::Request::create(computeServer,
					"add", "testrep", env);
	if ( env.check_exception() ) {
		cout << "Error creating request. " << endl;
		cout << env;
		return 0;
	}
	computeReq->bind();
	reqEnv = computeReq->_environment();
	if ( reqEnv.check_exception()) {
		cout << "Unable to bind to ComputeServer" << endl;
		cout << reqEnv;
		return(0);
	}
	else 
		cout << "Bound to ComputeServer" << endl;
	*computeReq->arg("a") = (float)atof(argv[1]);
	*computeReq->arg("b") = (float)atof(argv[2]);

	computeReq->invoke();
	if (reqEnv.check_exception() ) {
		cout << "Exception: " << reqEnv.exception_value()->_id()
			<< " raised." << endl;
	}
	CORBA::Float answer = (CORBA::Float)(*computeReq->result());
	cout << "Answer: " << answer << endl;

	return(1);

}



