#include <stdlib.h>
#include <signal.h>
#include "monsrv.h"


// ComputeServer/ Monitor client process has broken connection to this
// Object.
// Iterate over the server list and remove the compute servers implemented
// by this client
void MonitorServer::unbind(const CORBA::Principal& princ, const char *,
			const CORBA::Object& )
{
	CORBA::ULong num_servers = _servers.length();
	for (CORBA::ULong i = num_servers; i > 0; i--) {
		ServerAssoc& server = _servers[i-1];
		if ( server.host_name() == princ.hostName() &&
				server.pid() == princ.pid() )
			_servers.remove(i-1);
	}
}

void MonitorServer::add_user(const char *name)
{
	_userList.append(new CORBA::String(name));
}

CORBA::Boolean MonitorServer::valid_user(const CORBA::Principal *princ)
{
	const char *name = princ->userName();
	
	cout << "Checking validity for user " << name << endl;
	for (CORBA::ULong i=0; i< _userList.length(); i++) {
		if ( _userList[i] == name )
			return CORBA::TRUE;
	}
	return CORBA::FALSE;
}

void MonitorServer::register_server(ComputeServer *server)
{
	// Get client info
	const CORBA::Principal *princ = _principal();

	ServerAssoc *assoc = new ServerAssoc;
	assoc->host_name() = princ->hostName();
	assoc->pid() = princ->pid();
	assoc->server(server);
	
	// NOTE: We do not need to _duplicate the server 
	// (Increment the ref count) since append on sequences automatically
	// increment the ref count. The ref count is decrmented when
	// the element is removed/ sequence is deleted.
	_servers.append(assoc);
}

// Uses Round Robin scheduling to give out ComputeServer Objects
ComputeServer *MonitorServer::get_server(CORBA::Environment& env)
{
	if ( ! valid_user(_principal()) ) {
		env.exception_value(new PermissionDenied);
		return NULL;
	} 

	if ( _servers.length() == 0) {
		env.exception_value(new NoServers);
		return NULL;
	} 
	if ( _cur_index >= _servers.length() )
		_cur_index = 0;
	ComputeServer *server = _servers[_cur_index++].server();
	server->_duplicate();		// Return value is released by ORB
	return server;
}

void sighandler(int, ...)
{
	exit(0);
}


int main(int argc, char **argv)
{
	signal(SIGINT, sighandler);
	MonitorServer server("TestMonitor");
	int c;

	while ((c = getopt(argc, argv, "a:?")) != EOF) {
		switch (c) {
			case 'a':
				server.add_user(optarg);
				break;
			case '?':
			default:
				cout << "Usage: monsrv [-a user] [-a user] "
					<< endl;
				return 1;
		}
	}

	// Attach the event handler to implementation so that unbind
	// gets called when clients disconnect
	CORBA::Environment env;
	CORBA::BOA::instance()->event_handler(&server, env, &server);
	if ( env.check_exception() ) {
		cout << "Error registering event handler." << endl;
		cout << env;
	}

	CORBA::BOA::impl_is_ready();
	return 0;
}
			
	
