
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include "perf_s.hh"

int seq_size = 100;

class OnewayServer: public TestOneway_impl
{
    public:
	OnewayServer() {};
	~OnewayServer() {}

	void test_no_param() {}
	void test_prim_args(CORBA::Short, CORBA::Long, CORBA::Float, 
		CORBA::Double, CORBA::Char, const CORBA::String&) {}
	void test_struct(const PerfStruct& ) {}
	void test_prim_seq(const shortSeq&, const longSeq&, const floatSeq&,
		const doubleSeq&, const charSeq&, const stringSeq& ) {}
	void test_struct_seq(const structSeq& ) {}
	void test_struct_array(const structArray& ) {}
};

class ReqReplyServer: public TestReqReply_impl
{
    private:
	shortSeq	*_shortSeq;
	longSeq		*_longSeq;
	floatSeq	*_floatSeq;
	doubleSeq	*_doubleSeq;
	charSeq		*_charSeq;
	stringSeq	*_stringSeq;
	structSeq	*_structSeq;
	structArray	*_structArray;

    public:
	ReqReplyServer();
	~ReqReplyServer() { 
		_shortSeq->_release();
		_longSeq->_release();
		_floatSeq->_release();
		_doubleSeq->_release();
		_charSeq->_release();
		_stringSeq->_release();
		_structSeq->_release();
		_structArray->_release();
	}

	CORBA::Long test_prim_args(
			CORBA::Short, 
			CORBA::Long, 
			CORBA::Float, 
			CORBA::Double,
			CORBA::Char,
			const CORBA::String&, 
			CORBA::Short& ,
			CORBA::Long& ,
			CORBA::Float& ,
			CORBA::Double& ,
			CORBA::Char& ,
			CORBA::String& ,
			CORBA::Short& out_short,
			CORBA::Long& out_long,
			CORBA::Float& out_float,
			CORBA::Double& out_double,
			CORBA::Char& out_char,
			CORBA::String *&out_string) {
		
		out_short = 2;
		out_long = 2;
		out_float = 2.0;
		out_double = 2.0;
		out_char = 2;
		out_string = new CORBA::String("string");
	
		return 1;
	}

	CORBA::Long test_struct_args(const PerfStruct&, 
			PerfStruct&, 
			PerfStruct *&out_struct)
	{
		out_struct = new PerfStruct;
		return 1;
	}

	CORBA::Long test_prim_seq(
			const shortSeq&,
			const longSeq&,
			const floatSeq&,
			const doubleSeq&,
			const charSeq&,
			const stringSeq&,

			shortSeq&,
			longSeq&,
			floatSeq&,
			doubleSeq&,
			charSeq&,
			stringSeq&,

			shortSeq *& out_short,
			longSeq *& out_long,
			floatSeq *& out_float,
			doubleSeq *& out_double,
			charSeq *& out_char,
			stringSeq *& out_string) {
		out_short = _shortSeq;
		out_long = _longSeq;
		out_float = _floatSeq;
		out_double = _doubleSeq;
		out_char = _charSeq;
		out_string = _stringSeq;

		// ORB core releases return/ out parameters
		// Objects are duplicated so that they are not deleted
		_shortSeq->_duplicate();
		_longSeq->_duplicate();
		_floatSeq->_duplicate();
		_doubleSeq->_duplicate();
		_charSeq->_duplicate();
		_stringSeq->_duplicate();

		return 1;
	}

	CORBA::Long test_struct_seq(const structSeq&, structSeq&,
			structSeq *& out_seq) {
		out_seq = _structSeq;
		_structSeq->_duplicate();
		return 1;
	}

	CORBA::Long test_struct_array(const structArray&, structArray&,
			structArray *& out_array) {
		out_array = _structArray;
		_structArray->_duplicate();
		return 1;
	}
};


ReqReplyServer::ReqReplyServer()
{
	_shortSeq = new shortSeq;
	_shortSeq->_duplicate();
	_longSeq = new longSeq;
	_longSeq->_duplicate();
	_floatSeq = new floatSeq;
	_floatSeq->_duplicate();
	_doubleSeq = new doubleSeq;
	_doubleSeq->_duplicate();
	_charSeq = new charSeq;
	_charSeq->_duplicate();
	_stringSeq = new stringSeq;
	_stringSeq->_duplicate();
	_structSeq = new structSeq;
	_structSeq->_duplicate();
	_structArray = new structArray;
	_structArray->_duplicate();
	for (int i=0; i< seq_size; i++) {
		_shortSeq->append(0);
		_longSeq->append(0);
		_floatSeq->append(0);
		_doubleSeq->append(0);
		_charSeq->append(0);
		_stringSeq->append(new CORBA::String("String"));
		_structSeq->append(new PerfStruct);
	}
}


#ifdef SOLARIS
extern "C" int gettimeofday(struct timeval *);
#define GETTIME(x)	gettimeofday(x)
#else
extern "C" int gettimeofday(struct timeval *, struct timezone *);
#define GETTIME(x)	gettimeofday(x, (struct timezone *)NULL)
#endif


#define SIZE_STRUCT	(sizeof(short) + sizeof(long) + sizeof(float) + \
			sizeof(double) + sizeof(char) + strlen("String") )

#define NUM_ELEMENTS    100


struct timeval starttime;
struct timeval endtime;

int	num_samples = 100;

double total_time = 0.0;


void INIT_TIME()
{
	total_time = 0;
}

void START_TIME()
{
	GETTIME(&starttime);
}

void ADD_TIME()
{
	GETTIME(&endtime);
	total_time += ((endtime.tv_sec - starttime.tv_sec) * 1000.0) +
                        ((endtime.tv_usec - starttime.tv_usec)/ 1000.0);
}

void print_stats(const char *func_name, int num_bytes_per_sample) 
{

	cout << "\n\n----------- " << func_name << " ----------------" << endl;

	cout << "Average time in millisecs: " << (total_time / num_samples)
						<< endl; 
}

void test_no_param(TestOneway *oneway)
{
	int i;

	INIT_TIME();
	START_TIME();
	for (i=0; i< num_samples; i++)
		oneway->test_no_param();
	ADD_TIME();
	print_stats("Oneway::test_no_param", 0);
}

void test_prim_args(TestOneway *oneway)
{
	CORBA::Short shortVal = 0;
	CORBA::Long longVal = 0;
	CORBA::Float floatVal = 0.0;
	CORBA::Double doubleVal = 0.0;
	CORBA::Char charVal = 0;
	CORBA::String stringVal("String");

	INIT_TIME();
	START_TIME();
	for (int i=0; i< num_samples; i++)
		oneway->test_prim_args(shortVal, longVal, floatVal, doubleVal,
			charVal, stringVal);
	ADD_TIME();	
	int num_bytes = sizeof(short) + sizeof(long) +
			sizeof(float) + sizeof(double) +
			sizeof(char) + strlen("String"); 

	print_stats("Oneway::test_prim_args", num_bytes);
}

void test_struct(TestOneway *oneway)
{
	PerfStruct structVal;
	
	INIT_TIME();
	START_TIME();
	for (int i=0; i< num_samples; i++)
		oneway->test_struct(structVal);
	ADD_TIME();
	
	print_stats("Oneway::test_struct", SIZE_STRUCT);
}
	
void test_prim_seq(TestOneway *oneway)
{
	shortSeq shortVal;
	longSeq longVal;
	floatSeq floatVal;
	doubleSeq doubleVal;
	charSeq charVal;
	stringSeq stringVal;

	for (int i=0; i< seq_size; i++) {
		shortVal.append(0);
		longVal.append(0);
		floatVal.append(0);
		doubleVal.append(0);
		charVal.append(0);
		stringVal.append(new CORBA::String("String"));
	}

	INIT_TIME();
	START_TIME();
	for (i=0; i< num_samples; i++)
		oneway->test_prim_seq(shortVal, longVal, floatVal, doubleVal,
			charVal, stringVal);
	ADD_TIME();	
	int num_bytes = (sizeof(short) + sizeof(long) +
			sizeof(float) + sizeof(double) +
			sizeof(char) + strlen("String")) * seq_size;

	print_stats("Oneway::test_prim_seq", num_bytes);
}

	
void test_struct_seq(TestOneway *oneway)
{
	structSeq structVal;
	for (int i=0; i< seq_size; i++) {
		PerfStruct *val = new PerfStruct;
		val->stringVal() = "String";
		structVal.append(val);
	}

	INIT_TIME();
	START_TIME();
	for (i=0; i< num_samples; i++)
		oneway->test_struct_seq(structVal);
	ADD_TIME();
	
	int num_bytes = SIZE_STRUCT * seq_size;
	print_stats("Oneway::test_struct_seq", num_bytes);
	
}
	
void test_struct_array(TestOneway *oneway)
{
	structArray structVal;

	for (int i=0; i< NUM_ELEMENTS; i++)
		structVal[i].stringVal() = "String";

	INIT_TIME();
	START_TIME();
	for (i=0; i< num_samples; i++)
		oneway->test_struct_array(structVal);
	ADD_TIME();
	
	int num_bytes = SIZE_STRUCT * NUM_ELEMENTS;
	print_stats("Oneway::test_struct_array", num_bytes);
}


void test_prim_args(TestReqReply *obj)
{
	CORBA::Short in_short=0;
	CORBA::Long  in_long=0;
	CORBA::Float in_float = 0.0;
	CORBA::Double in_double = 0.0;
	CORBA::Char   in_char = 0;
	CORBA::String in_string("String");

	CORBA::Short inout_short=0;
	CORBA::Long  inout_long=0;
	CORBA::Float inout_float = 0.0;
	CORBA::Double inout_double = 0.0;
	CORBA::Char   inout_char = 0;
	CORBA::String inout_string("String");
	
	CORBA::Short out_short = 0;
	CORBA::Long  out_long = 0;
	CORBA::Float out_float = 0;
	CORBA::Double out_double = 0;
	CORBA::Char   out_char = 0;
	CORBA::String *out_string = NULL;

	CORBA::Long ret;

	INIT_TIME();
	for (int i=0; i< num_samples; i++)  {
		START_TIME();
		ret = obj->test_prim_args(in_short, in_long, in_float, 
			in_double, in_char, in_string,
			inout_short, inout_long, inout_float, inout_double,
			inout_char, inout_string,
			out_short, out_long, out_float, out_double,
			out_char, out_string);
		ADD_TIME();
		out_string->_release();
	}
	// Num of bytes sent/received
	int num_bytes = 4 * (sizeof(short) + sizeof(long) + sizeof(float) +
				sizeof(double) + sizeof(char) 
				+ strlen("String"));
	print_stats("TestReqReply::test_prim_args", num_bytes);
}

void test_struct_args(TestReqReply *obj)
{
	struct PerfStruct in_struct;
	struct PerfStruct *out_struct = NULL;
	struct PerfStruct inout_struct;

	in_struct.stringVal() = "String";
	inout_struct.stringVal() = "String";

	INIT_TIME();
	for (int i=0; i< num_samples; i++) {
		START_TIME();
		obj->test_struct_args(in_struct, inout_struct, out_struct);
		ADD_TIME();
		out_struct->_release();
	}

	// Num of bytes sent/received
	int num_bytes = 4 * SIZE_STRUCT;
	print_stats("TestReqReply::test_struct_args", num_bytes);
}

void test_prim_seq(TestReqReply *obj)
{
	shortSeq	in_short;
	longSeq		in_long;
	floatSeq	in_float;
	doubleSeq	in_double;
	charSeq		in_char;
	stringSeq	in_string;

	shortSeq	inout_short;
	longSeq		inout_long;
	floatSeq	inout_float;
	doubleSeq	inout_double;
	charSeq		inout_char;
	stringSeq	inout_string;

	shortSeq	*out_short = NULL;
	longSeq		*out_long = NULL;
	floatSeq	*out_float = NULL;
	doubleSeq	*out_double = NULL;
	charSeq		*out_char = NULL;
	stringSeq	*out_string = NULL;
	
	for (int i=0; i < seq_size; i++) {
		in_short.append(0);
		in_long.append(0);
		in_float.append(0);
		in_double.append(0);
		in_char.append(0);
		in_string.append(new CORBA::String("String"));
		
		inout_short.append(0);
		inout_long.append(0);
		inout_float.append(0);
		inout_double.append(0);
		inout_char.append(0);
		inout_string.append(new CORBA::String("String"));
	}

	CORBA::Long ret;
	INIT_TIME();
	for (i=0; i< num_samples; i++) {
		START_TIME();
		ret = obj->test_prim_seq(in_short, in_long, in_float, in_double,
				in_char, in_string,
				inout_short, inout_long, inout_float,
				inout_double, inout_char, inout_string,
				out_short, out_long, out_float, out_double,
				out_char, out_string);
		ADD_TIME();
		out_short->_release();
		out_long->_release();
		out_float->_release();
		out_double->_release();
		out_char->_release();
		out_string->_release();
	}

	// Num of bytes sent/received
	int num_bytes = 4 * seq_size * (sizeof(short) + sizeof(long) +
			sizeof(float) + sizeof(double) + sizeof(char) +
			strlen("String"));

	print_stats("TestReqReply::test_prim_seq", num_bytes);
}

void test_struct_seq(TestReqReply *obj)
{
	structSeq	in_struct;
	structSeq	inout_struct;
	structSeq	*out_struct = NULL;

	for (int i=0; i< seq_size; i++) {
		PerfStruct *in = new PerfStruct;
		PerfStruct *inout = new PerfStruct;
		in->stringVal() = "String";
		inout->stringVal() = "String";
		in_struct.append(new PerfStruct);
		inout_struct.append(new PerfStruct);
	}
	CORBA::Long ret;
	INIT_TIME();
	for (i=0; i< num_samples; i++) {
		START_TIME();
		ret = obj->test_struct_seq(in_struct, inout_struct, out_struct);
		ADD_TIME();
		out_struct->_release();
	}

	// Num of bytes sent/received
	int num_bytes = 4 * seq_size * SIZE_STRUCT;
	
	print_stats("TestReqReply::test_struct_seq", num_bytes);
}

void test_struct_array(TestReqReply *obj)
{
	structArray	in_struct;
	structArray	inout_struct;
	structArray	*out_struct = NULL;

	for (int i=0; i< NUM_ELEMENTS; i++) {
		in_struct[i].stringVal() = "String";
		inout_struct[i].stringVal() = "String";
	}

	CORBA::Long ret;

	INIT_TIME();
	for (i=0; i< num_samples; i++) {
		START_TIME();
		ret = obj->test_struct_array(in_struct, inout_struct, 
			out_struct);
		ADD_TIME();
		out_struct->_release();
	}

	// Num of bytes sent/received
	int num_bytes = 4 * NUM_ELEMENTS * SIZE_STRUCT;

	print_stats("TestReqReply::test_struct_array", num_bytes);
}

void sighandler(int, ...)
{
	exit(0);
}

	
int main(int argc, char **argv)
{
	signal(SIGINT, sighandler);
	int c;
	while ((c = getopt(argc, argv, "n:s:?")) != EOF) {
		switch (c) {
			case 'n':
				num_samples = atoi(optarg);
				break;
			case 's':
				seq_size = atoi(optarg);
				break;
			case '?':
			default:
				cout << "Usage: perfclnt [-n num_samples] "
					<< "[-s seq_size]" << endl;
				return 0;
		}
	}
	// Construct oneway and reqreply servers
	OnewayServer server;
	ReqReplyServer reqReply;
	
	TestOneway *oneway = TestOneway::_bind();
	if ( !oneway ) {
		cout << "testOneway: Unable to connect to server. "
			<< endl;
		return 0;
	}
	test_no_param(oneway);
	test_prim_args(oneway);
	test_struct(oneway);
	test_prim_seq(oneway);
	test_struct_seq(oneway);
	test_struct_array(oneway);
	oneway->_release();


	TestReqReply *obj = TestReqReply::_bind();
	if ( !obj ) {
		cout << "TestReqReply: Unable to connect to server. "
			<< endl;
		return 0;
	}
	test_prim_args(obj);
	test_struct_args(obj);
	test_prim_seq(obj);
	test_struct_seq(obj);
	test_struct_array(obj);

	obj->_release();

	return 0;
}

