#ifndef _mstrm_h
#define _mstrm_h
/****************************************************************************/
/*                                                                          */
/*  ORBeline (c)   is copyrighted by PostModern Computing Technologies, Inc */
/*                                                                          */
/*  Copyright 1993, 1994 by PostModern Computing Technologies, Inc.         */
/*                          1897 Landings Drive                             */
/*                          Mountain View, California, USA                  */
/*                          Telephone (415) 967-6169                        */
/*                          Facsimile (415) 967-6212                        */
/*                                                                          */
/*  This code is the sole property of PostModern Computing Technologies,    */
/*  Inc.  and is  protected  by  copyright under the laws of the United     */
/*  States.   This  program is confidential, proprietary, and  a  trade     */
/*  secret,  not  to  be disclosed  without written authorization  from     */
/*  PostModern Computing Technologies, Inc.  Any use,  duplication,  or     */
/*  disclosure  of  this  program  by other than  PostModern  Computing     */
/*  Technologies, Inc. and its assigned licensees is strictly forbidden     */
/*  by law.                                                                 */
/*                                                                          */
/*  All rights reserved.                                                    */
/*                                                                          */
/*  Author: psm                                                             */
/*                                                                          */
/****************************************************************************/

// Base class for all Marshalling/Unmarshalling streams
class MarshalStream: public NCistream, public NCostream
{
    protected:
	static const char	*_class_name;
	CORBA::Environment	_env;
	MarshalStream() {}
    public:
	enum ArgMode {
		ARG_IN,
		ARG_OUT,
		ARG_INOUT,
		ARG_RETURN
	};
	virtual ~MarshalStream() {}

	void	clearEnv() { _env.clear_exception(); }
	const CORBA::Environment& env() const { return _env; }
	CORBA::Boolean check_exception()  const { 
			return _env.check_exception();
	}
	CORBA::Exception *exception_value() {
		return _env.exception_value();
	}

	virtual MarshalStream& rewindInBuffer(CORBA::Environment& env) = 0;
	virtual MarshalStream& rewindOutBuffer(CORBA::Environment& env) = 0;
	virtual MarshalStream& rewindBuffer(CORBA::Environment& env) {
		rewindInBuffer(env); 
		if ( !env.check_exception() )
			return rewindOutBuffer(env); 
		else
			return *this;
	}
	virtual MarshalStream& flush(CORBA::Environment& env) { 
			if ( _env.check_exception() )
				env.exception_value(_env.exception_value());
			return *this; 
	}
	

	virtual MarshalStream& beginStructure(ArgMode ) { return *this; }
	virtual MarshalStream& endStructure(ArgMode ) { return *this; }

	virtual MarshalStream& beginArray(ArgMode ) { return *this; }
	virtual MarshalStream& endArray(ArgMode ) { return *this; }

	virtual MarshalStream& beginSequence(ArgMode ) { return *this; }
	virtual MarshalStream& endSequence(ArgMode ) { return *this; }

	virtual MarshalStream& beginUnion(ArgMode ) { return *this; }
	virtual MarshalStream& endUnion(ArgMode ) { return *this; }

	virtual MarshalStream&  sendMessage( Environment& env ) = 0;
	virtual MarshalStream&  receiveMessage(Environment& env,
			CORBA::ULong timeout_secs = 0) = 0;
	virtual MarshalStream&  sendAndWaitForReply(Environment& env,
			CORBA::ULong timeout_secs = 0) = 0;

	virtual MarshalStream& putShort(const Short& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putLong(const Long& arg, 
					ArgMode = ARG_IN) =0;
	virtual MarshalStream& putUShort(const UShort& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putULong(const ULong& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putChar(const Char& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putFloat(const Float& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putDouble(const Double& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putOctet(const Octet& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putBoolean(const Boolean& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putObjRef(Object *obj, 
					const TypeInfo *expectedType,
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putString(const String& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putString(String *arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putStructure(const Structure& obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putStructure(Structure *obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putArray(const Array& obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putArray(Array *obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putUnion(const Union& obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putUnion(Union *obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putSequence(const Sequence& obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putSequence(Sequence *obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putEnvironment(const Environment& obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putEnvironment(Environment *obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putPrincipal(const Principal& obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putPrincipal(Principal *obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putContext(const Context& obj,
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putContext(Context *obj,
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& putAny(const Any& obj,
					ArgMode = ARG_IN) =0;
	virtual MarshalStream& putAny(Any *obj,
					ArgMode = ARG_IN) =0;
	virtual MarshalStream& putAnyValue(const Any& obj, 
					ArgMode = ARG_IN) =0;
	virtual MarshalStream& getShort(Short& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getLong(Long& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getUShort(UShort& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getULong(ULong& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getChar(Char& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getFloat(Float& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getDouble(Double& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getOctet(Octet& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getBoolean(Boolean& arg, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getObjRef(Object *& obj, 
					const TypeInfo *expected_type,
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getString(String& arg, 
					ArgMode = ARG_IN, 
					void *ptr=(void *)NULL) = 0;
	virtual MarshalStream& getStructure(Structure& obj, 
					ArgMode = ARG_IN, 
					void *ptr=(void *)NULL) = 0;
	virtual MarshalStream& getArray(Array& obj, 
					ArgMode = ARG_IN, 
					void *ptr=(void *)NULL) = 0;
	virtual MarshalStream& getSequence(Sequence& obj, 
					ArgMode = ARG_IN, 
					void *ptr=(void *)NULL) = 0;
	virtual MarshalStream& getUnion(Union& obj, 
					ArgMode = ARG_IN, 
					void *ptr=(void *)NULL) = 0;
	virtual MarshalStream& getEnvironment(Environment& obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getPrincipal(Principal& obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getContext(Context& obj, 
					ArgMode = ARG_IN) = 0;
	virtual MarshalStream& getAny(Any& obj, 
					ArgMode = ARG_IN, 
					void *ptr=(void *)NULL) = 0;
	virtual MarshalStream& getAnyValue(Any& obj, 
					ArgMode = ARG_IN) = 0;

	virtual MarshalStream& writeObjRef(const CORBA::Object *obj,
				const CORBA::TypeInfo *expectedType) = 0;
	virtual MarshalStream& readObjRef(CORBA::Object *& obj,
				const CORBA::TypeInfo *expectedType) =0;

	static const char *desc();
	virtual const char *className() const;
	virtual NCbool isKindOf(const char *name) const;

};


#endif
