// "@(#)Module Name: nciostream.h Version:2.1 Date: 94/06/19"
#ifndef _nciostream_h
#define _nciostream_h
/****************************************************************************/
/*                                                                          */
/*  ORBeline (c)   is copyrighted by PostModern Computing Technologies, Inc */
/*                                                                          */
/*  Copyright 1993, 1994 by PostModern Computing Technologies, Inc.         */
/*                          1897 Landings Drive                             */
/*                          Mountain View, California, USA                  */
/*                          Telephone (415) 967-6169                        */
/*                          Facsimile (415) 967-6212                        */
/*                                                                          */
/*  This code is the sole property of PostModern Computing Technologies,    */
/*  Inc.  and is  protected  by  copyright under the laws of the United     */
/*  States.   This  program is confidential, proprietary, and  a  trade     */
/*  secret,  not  to  be disclosed  without written authorization  from     */
/*  PostModern Computing Technologies, Inc.  Any use,  duplication,  or     */
/*  disclosure  of  this  program  by other than  PostModern  Computing     */
/*  Technologies, Inc. and its assigned licensees is strictly forbidden     */
/*  by law.                                                                 */
/*                                                                          */
/*  All rights reserved.                                                    */
/*                                                                          */
/****************************************************************************/
#include <stdio.h>
#include <iostream.h>

// -------------------------------------------------------------------------
// Base class for NetClasses I/O stream implementation. This class defines
// the operations that can be performed on NetClasses I/O stream classes.
// -------------------------------------------------------------------------
class NCTransObject;
class NCTypeInfo;
class NCInTbl;

#ifndef _NCbool
#define _NCbool
typedef int NCbool;
#define NC_YES 1
#define NC_NO  0
#endif

class NCistream 
{
    private:
	istream			*_istrm;
	static const char	*_class_name;
    protected:
	NCInTbl			*_inTbl;

	NCistream();
	virtual NCTransObject* readObject(const NCTypeInfo *classinfo);
	virtual void clearInClassTbl();
	virtual void clearInObjTbl();
	void clearInTbl() { clearInClassTbl(); clearInObjTbl(); }
    public:
	NCistream(istream& istrm);
	virtual ~NCistream();
	virtual NCTransObject* readTransObj();
	virtual void addToTbl(NCTransObject& ); 
	static const char *desc() { return _class_name; }
	virtual  const char *className() const { return _class_name; }
	virtual  NCbool isKindOf(const char *name) const { 
			return (name == _class_name);
	}
// NOTE: The following operator reads characters from stream until a NULL or
//	 newlinine terminator is encountered. Make sure you have enough memory 
//	 allocated for the 'c'. 
	virtual NCistream& operator>>(char* c);
	virtual NCistream& operator>>(char& c);
#if !defined(CRAY)
	virtual NCistream& operator>>(unsigned char& c);
#endif
	virtual NCistream& operator>>(short& x);
	virtual NCistream& operator>>(unsigned short& x);
	virtual NCistream& operator>>(int& x);
	virtual NCistream& operator>>(unsigned int& x); 
	virtual NCistream& operator>>(long& x); 
	virtual NCistream& operator>>(unsigned long& x); 
	virtual NCistream& operator>>(float& x); 
	virtual NCistream& operator>>(double& x); 

	virtual int get(); 
	virtual NCistream& get(char& c); 
#if !defined(CRAY)
	virtual NCistream& get(unsigned char& c); 
#endif
	virtual NCistream& get(char* c, unsigned size); 
#if !defined(CRAY)
	virtual NCistream& get(unsigned char* c, unsigned size); 
#endif
	virtual NCistream& get(short* x, unsigned size); 
	virtual NCistream& get(unsigned short* x, unsigned size); 
	virtual NCistream& get(int* x, unsigned size); 
	virtual NCistream& get(unsigned int* x, unsigned size); 
	virtual NCistream& get(long* x, unsigned size); 
	virtual NCistream& get(unsigned long* x, unsigned size); 
	virtual NCistream& get(float* x, unsigned size); 
	virtual NCistream& get(double* x, unsigned size); 
	virtual NCistream& getCString(char* c, unsigned maxlen); 
};

class NCOutTbl;
class NCostream 
{
    private:
	static const char	*_class_name;
	ostream			*_ostrm;
    protected:
	NCOutTbl	*_outTbl;

	NCostream();
	virtual void storeObject(const NCTransObject&);
	virtual void clearOutClassTbl();
	virtual void clearOutObjTbl();
	void clearOutTbl() { clearOutClassTbl(); clearOutObjTbl(); }
    public:
	NCostream(ostream& ostrm);
	virtual ~NCostream();
	static const char *desc() { return _class_name; }
	virtual  const char *className() const { return _class_name; }
	virtual  NCbool isKindOf(const char *name) const { 
			return (name == _class_name);
	}

	virtual void writeTransObj(const NCTransObject* obj);

// NOTE: The following writes characters to stream until a NULL terminator
//	 is encountered.
	virtual NCostream& operator<<(const char* c); 
	virtual NCostream& operator<<(char c); 
#if !defined(CRAY)
	virtual NCostream& operator<<(unsigned char c); 
#endif
	virtual NCostream& operator<<(short x); 
	virtual NCostream& operator<<(unsigned short x); 
	virtual NCostream& operator<<(int x); 
	virtual NCostream& operator<<(unsigned x); 
	virtual NCostream& operator<<(long x); 
	virtual NCostream& operator<<(unsigned long x); 
	virtual NCostream& operator<<(float x); 
	virtual NCostream& operator<<(double x); 
	virtual NCostream& operator<<(const NCTransObject* obj); 
	virtual NCostream& operator<<(const NCTransObject& obj); 

	virtual NCostream& put(char c); 
	virtual NCostream& put(const char* val, unsigned size); 
#if !defined(CRAY)
	virtual NCostream& put(const unsigned char* val, unsigned size); 
#endif
	virtual NCostream& put(const short* val, unsigned size); 
	virtual NCostream& put(const unsigned short* val, unsigned size); 
	virtual NCostream& put(const int* val, unsigned size); 
	virtual NCostream& put(const unsigned int* val, unsigned size); 
	virtual NCostream& put(const long* val, unsigned size); 
	virtual NCostream& put(const unsigned long* val, unsigned size); 
	virtual NCostream& put(const float* val, unsigned size); 
	virtual NCostream& put(const double* val, unsigned size); 
	virtual NCostream& putCString(const char* val); 

	NCostream& operator<< (NCostream& (*f)(NCostream&)) {
		return (*f)(*this);
	}
};

NCostream& endl(NCostream& strm);

#endif
