#ifndef _object_h_
#define _object_h_
/****************************************************************************/
/*                                                                          */
/*  ORBeline (c)   is copyrighted by PostModern Computing Technologies, Inc */
/*                                                                          */
/*  Copyright 1993, 1994 by PostModern Computing Technologies, Inc.         */
/*                          1897 Landings Drive                             */
/*                          Mountain View, California, USA                  */
/*                          Telephone (415) 967-6169                        */
/*                          Facsimile (415) 967-6212                        */
/*                                                                          */
/*  This code is the sole property of PostModern Computing Technologies,    */
/*  Inc.  and is  protected  by  copyright under the laws of the United     */
/*  States.   This  program is confidential, proprietary, and  a  trade     */
/*  secret,  not  to  be disclosed  without written authorization  from     */
/*  PostModern Computing Technologies, Inc.  Any use,  duplication,  or     */
/*  disclosure  of  this  program  by other than  PostModern  Computing     */
/*  Technologies, Inc. and its assigned licensees is strictly forbidden     */
/*  by law.                                                                 */
/*                                                                          */
/*  All rights reserved.                                                    */
/*                                                                          */
/****************************************************************************/

class Object
{
	friend _ObjectImpl;
	friend CORBA::ORB;
    public:
	ULong 			_ref_count() const;
	void			_ref();
	void			_release();
	Object			*_duplicate() const;
	Object			*_clone() const;
	Boolean 		_is_nil() const;

	virtual Boolean		_is_local() const;
	Boolean			_is_remote() const { return !_is_local(); }
	Boolean			_is_bound() const;
	Boolean			_is_persistent() const;

	const char		*_object_name() const;
	virtual const char	*_interface_name() const;

	const Principal 	*_principal() const;
	void			_principal(const Principal& principal);
	void			_principal(Principal *principal);

	static const Principal	*_default_principal();
	static void		_default_principal(const CORBA::Principal& );

	const BindOptions	*_bind_options() const;
	static const BindOptions *_default_bind_options();
	static void		_default_bind_options(
					const CORBA::BindOptions& );

	_ObjectImpl		*_object_impl() { return _impl; }
	const _ObjectImpl	*_object_impl() const { return _impl; }

	Environment&		_environment();
	

	// Functions to support narrowing and transporting
	static const TypeInfo *_desc() { return &_class_info; }
	virtual const TypeInfo *_type_info() const {
			return _desc();
	}
	virtual void 	     		_writer(NCostream& ) const;
	static CORBA::Object	*_reader(NCistream& strm); 
	static CORBA::Object	*_read(NCistream& strm, 
				const TypeInfo *expectedType);
	static void		_write(const CORBA::Object *obj, 
				NCostream& strm, const TypeInfo *expectedType);

	virtual void *_safe_narrow(const TypeInfo *desc) const;
    private:
	static const 	TypeInfo _class_info;
	_ObjectImpl		*_impl;
    protected:
	Object(const Object& ) {}
	void operator=(const Object& ) {}
	virtual			~Object();

	Object(const char *object_name = (const char *)NULL, 
				ORB::_ORBType type = ORB::_ORB_POSTMODERN);
	Object(NCistream& strm);
	void			_object_name(const char *name);

	void			_bind(const char *interface_name,
					CORBA::Environment& env,
					const char *object_name = 
						(const char *)NULL,
					const char *host_name = 
						(const char *)NULL,
					const BindOptions *options =
						(const BindOptions *)NULL);

	void			_rebind(CORBA::Environment& env);

	void			_unbind(CORBA::Environment& env);

	MarshalStream		*_create_oneway_request(
					const char *interface_name,
					CORBA::ULong method_id,
					CORBA::Environment& env);

	MarshalStream		*_create_invoke_request(
					const char *interface_name,
					CORBA::ULong method_id,
					CORBA::Environment& env);

	void			_send_oneway(CORBA::Environment& env);

	MarshalStream		*_invoke(CORBA::Environment& env);

	void			_orb_type(ORB::_ORBType 
					type=ORB::_ORB_POSTMODERN);

	void			_register_implementation(
					const char *interface_name,
					CORBA::ULong num_methods,
					_PMCSkelFunc *func_list,
					void *user_data,
					Environment& env,
					BOA::RegistrationScope scope,
					BOA::IMPLEventHandler *handler=
						(BOA::IMPLEventHandler *)NULL);
	void			_register_implementation(
					const char *interface_name,
					CORBA::ULong num_methods,
					_PMCSkelFunc *func_list,
					void *user_data,
					Environment& env,
					BOA::IMPLEventHandler *handler=
						(BOA::IMPLEventHandler *)NULL) {
				_register_implementation(interface_name,
					num_methods, func_list, user_data,
					env, BOA::scope(), handler);
	}
					

	void			_unregister_implementation(
					const char *interface_name,
					void *user_data,
					Environment& env);

	static CORBA::Object	*_implementation(
					const char *interface_name,
					const char *object_name = 
						(const char *)NULL);
    public:
	virtual void		_receive_reply(CORBA::MarshalStream& ,
						CORBA::Environment& ) {}
	virtual void		_exception(CORBA::Environment& env);
};

typedef Object *ObjectRef;
#endif
	
