#ifndef _typecode_h
#define _typecode_h
/****************************************************************************/
/*                                                                          */
/*  ORBeline (c)   is copyrighted by PostModern Computing Technologies, Inc */
/*                                                                          */
/*  Copyright 1993, 1994 by PostModern Computing Technologies, Inc.         */
/*                          1897 Landings Drive                             */
/*                          Mountain View, California, USA                  */
/*                          Telephone (415) 967-6169                        */
/*                          Facsimile (415) 967-6212                        */
/*                                                                          */
/*  This code is the sole property of PostModern Computing Technologies,    */
/*  Inc.  and is  protected  by  copyright under the laws of the United     */
/*  States.   This  program is confidential, proprietary, and  a  trade     */
/*  secret,  not  to  be disclosed  without written authorization  from     */
/*  PostModern Computing Technologies, Inc.  Any use,  duplication,  or     */
/*  disclosure  of  this  program  by other than  PostModern  Computing     */
/*  Technologies, Inc. and its assigned licensees is strictly forbidden     */
/*  by law.                                                                 */
/*                                                                          */
/*  All rights reserved.                                                    */
/*                                                                          */
/****************************************************************************/

class TypeCode: public Object
{	
    public:
	// CORBA Compliant interfaces
	enum TCKind {
		tk_null,
		tk_void,
		tk_short,
		tk_long,
		tk_ushort,
		tk_ulong,
		tk_float,
		tk_double,
		tk_boolean,
		tk_char,
		tk_octet,
		tk_any,
		tk_TypeCode,
		tk_Principal,
		tk_objref,
		tk_struct,
		tk_union,
		tk_enum,
		tk_string,
		tk_sequence,
		tk_array
	};
	class Bounds: public CORBA::UserException
	{
		DECLARE_NCOBJECT_CLASS(Bounds);
	    protected:
                void    veryPrettyPrint(NCostream& strm, int spaces=0) const;
		~Bounds() {}
            public: 
		Bounds() {}
	};

	TypeCode() { _kind = tk_null; }
	TypeCode(TCKind kind);
	TCKind	kind() const { return _kind; }
	Long param_count() const { return (Long)_parameters.size(); }
        Boolean equal(const TypeCode& tc) const {
		return (_kind == tc._kind && 
			_parameters.isEqual(tc._parameters));
	}
	Any	*parameter(Long index, Environment& env ) const;



	static const TypeCode		TC_null;
	static const TypeCode		TC_void;
	static const TypeCode		TC_short;
	static const TypeCode		TC_long;
	static const TypeCode		TC_ushort;
	static const TypeCode		TC_ulong;
	static const TypeCode		TC_float;
	static const TypeCode		TC_double;
	static const TypeCode		TC_boolean;
	static const TypeCode		TC_char;
	static const TypeCode		TC_octet;
	static const TypeCode		TC_any;
	static const TypeCode		TC_TypeCode;
	static const TypeCode		TC_Principal;

	virtual ~TypeCode() {};

	virtual Boolean is_primitive() const { return CORBA::TRUE; }
	static const char *convert_to_string(TCKind kind);

	static TypeCode *_read(NCistream& strm);
	// A virtual copy constructor. Constructs an instance of the
	// right typecode.
	static TypeCode *_create(const TypeCode& code);

        virtual void veryPrettyPrint(NCostream& strm, int spaces=0) const;

    protected:
	TCKind			_kind;
	DSOrderedCltn           _parameters;
	
	void		 	_read_parameters(NCistream& strm);

	void operator=(const TypeCode&);
    private:
	static const CORBA::TypeInfo	_class_info;
    public:
	TypeCode(NCistream& strm);

	static const CORBA::TypeInfo *_desc();
	virtual const CORBA::TypeInfo *_type_info() const;
	virtual void *_safe_narrow(const CORBA::TypeInfo *)const;
	static TypeCode *_narrow(const CORBA::Object *obj);
	static CORBA::Object *_reader(NCistream& strm) {
		return TypeCode::_read(strm);
	}
	void _writer(NCostream& strm) const;
	Boolean	_is_local() const { return CORBA::TRUE; }
	int compare(const TypeCode& tc) const;
	unsigned hash() const;	

	virtual const char *_interface_name() const { return "CORBA::TypeCode";}


};

// ------------------------------------------------------------------------
// StructTypeCode class
// ------------------------------------------------------------------------
class StructTypeCode: public TypeCode
{
    private:
	static const CORBA::TypeInfo	_class_info;
    public:
	static const StructTypeCode	TC_RepositoryDescription;
	static const StructTypeCode	TC_InterfaceDescription;
	static const StructTypeCode	TC_ModuleDescription;
	static const StructTypeCode     TC_OperationDescription;
	static const StructTypeCode     TC_ParameterDescription;
	static const StructTypeCode     TC_ExceptionDescription;
	static const StructTypeCode     TC_AttributeDescription;
	static const StructTypeCode     TC_TypeDescription;
	static const StructTypeCode     TC_ConstDescription;
	static const StructTypeCode	TC_FullInterfaceDescription;

        static const StructTypeCode     TC_Description;
        static const StructTypeCode     TC_DBKey;
        static const StructTypeCode     TC_DBRecord;      

	StructTypeCode(ValueString *struct_name, 
		       ValueString *mem1,
		       ValueTypeCode *tc1,
		       ... );		// The variable length argument
			                // list constructor for structures
					// expects repeating pairs of the form
					// (member-name, TypeCode)
					// after the initial struct_name.

        //
        // Also need a var arg constructor suitable for TypeCode constants
        //

	StructTypeCode(const char *struct_name, 
		       const char *mem1,
		       const TypeCode *tc1,
		       ... );		

        // Alternatively, can construct a StructTypeCode specifying
	// nothing.  If we use this constructor, it's our
        // responsibility to call add_parameter() repeated later 
        // to add the name followed by the necessary (member, type code) 
        // repeating pairs.
					 
	StructTypeCode() : TypeCode(TypeCode::tk_struct) {}
	StructTypeCode(NCistream& strm);

	~StructTypeCode() {}

	Boolean is_primitive() const { return CORBA::FALSE; }
	const char *name() const;
	const TypeCode& member_typecode(const char *member_name) const;
	const TypeCode& member_typecode(Long member_index) const;
	Long  member_count() const { return (Long)((_parameters.size() -1)/2); }
	const char *member_name(Long member_index) const;
	Long  member_index(const char *name) const;
	Boolean has_member(const char *name) const {
		return (member_index(name) >= 0);
	}

        void veryPrettyPrint(NCostream& strm, int spaces=0) const;
	void add_parameter(Value* parameter);

	static const CORBA::TypeInfo *_desc();
	virtual const CORBA::TypeInfo *_type_info() const;
	virtual void *_safe_narrow(const CORBA::TypeInfo *)const;
	static StructTypeCode *_narrow(const CORBA::Object *obj);
	const char *_interface_name() const { return "CORBA::StructTypeCode";}
};

// ------------------------------------------------------------------------
// UnionTypeCode class
// ------------------------------------------------------------------------
class UnionTypeCode: public TypeCode
{
    private:
	static const CORBA::TypeInfo	_class_info;
    public:
	UnionTypeCode(ValueString     * union_name, 
		      ValueTypeCode   * switch_type_code,
		      Value           * label_value,
		      ValueString     * member_name,
		      ValueTypeCode   * member_type_code,
		      ...);	      
	                                // The variable length argument
			                // list constructor expects
					// repeating triples of the form
					// (label-value, member-name, TypeCode)
					// after the initial struct_name.

        //
        // Also need a var arg constructor suitable for TypeCode constants
        //

	UnionTypeCode(const char      * union_name,
                      const TypeCode  * switch_type_code,
                      const char      * label_value,
                      const char      * member_name,
                      const TypeCode  * member_type_code,
		       ... );		

        // If we use this constructor, its our responsibility to
        // later call add_parameter to add the union_name, switch_type_code
        // and any repeating the triples.
 
	UnionTypeCode() : TypeCode(TypeCode::tk_union) {}
	UnionTypeCode(NCistream& strm);
	~UnionTypeCode() { }

	Boolean is_primitive() const { return CORBA::FALSE; }

        void veryPrettyPrint(NCostream& strm, int spaces=0) const;

	void add_parameter(Value* parameter);

	const char *name() const;
	const TypeCode& switch_typecode() const;
	const TypeCode& value_typecode(const Value& discrim_value) const;

	static const CORBA::TypeInfo *_desc();
	virtual const CORBA::TypeInfo *_type_info() const;
	virtual void *_safe_narrow(const CORBA::TypeInfo *)const;
	static UnionTypeCode *_narrow(const CORBA::Object *obj);
	const char *_interface_name() const { return "CORBA::UnionTypeCode";}
};

// ------------------------------------------------------------------------
// EnumTypeCode class
// ------------------------------------------------------------------------
class EnumTypeCode: public TypeCode
{
    private:
	static const CORBA::TypeInfo	_class_info;
    public:
        static const EnumTypeCode	TC_OperationMode;
        static const EnumTypeCode	TC_ParameterMode;
        static const EnumTypeCode	TC_AttributeMode;

	EnumTypeCode(ValueString* enum_name, 
		     ValueString* enum_id,
		     ... );		// The variable length argument
			                // list constructor for enumerations
					// expects a sequence of enumeration 
					// ids as paramters after the initial 
					// enum_name.

        //
        // Also need a var arg constructor suitable for TypeCode constants
        //

	EnumTypeCode(const char *enum_name,
                     const char *enum_id,
		       ... );		

        // Alternatively, can construct a EnumTypeCode specifying
	// only its name.  If we use this constructor, it's our
        // responsibility to call add_parameter() later to add the 
        // name and necessary enum_id's.
					 
	EnumTypeCode() : TypeCode(TypeCode::tk_enum) {}

	EnumTypeCode(NCistream& strm);
	~EnumTypeCode() {}

	Boolean is_primitive() const { return CORBA::FALSE; }
        void veryPrettyPrint(NCostream& strm, int spaces=0) const;
	void add_parameter(Value* parameter);
	Long  member_count() const { return (Long)(_parameters.size() -1); }
	const char *name() const;
	const char *member_name(Long index) const;
	Long member_value(const char *member_name) const;


	static const CORBA::TypeInfo *_desc();
	virtual const CORBA::TypeInfo *_type_info() const;
	virtual void *_safe_narrow(const CORBA::TypeInfo *)const;
	static EnumTypeCode *_narrow(const CORBA::Object *obj);
	const char *_interface_name() const { return "CORBA::EnumTypeCode";}
};


// ------------------------------------------------------------------------
// StringTypeCode class
// ------------------------------------------------------------------------
class StringTypeCode: public TypeCode
{
    private:
	static const CORBA::TypeInfo	_class_info;
    public:
	static const StringTypeCode		TC_string;

	StringTypeCode(Long maxlen);
	StringTypeCode(ValueLong *maxlen);
	StringTypeCode(NCistream& strm);

	Boolean is_primitive() const { return CORBA::FALSE; }
        void veryPrettyPrint(NCostream& strm, int spaces=0) const;
	~StringTypeCode() {}


	static const CORBA::TypeInfo *_desc();
	virtual const CORBA::TypeInfo *_type_info() const;
	virtual void *_safe_narrow(const CORBA::TypeInfo *)const;
	static StringTypeCode *_narrow(const CORBA::Object *obj);
	const char *_interface_name() const { return "CORBA::StringTypeCode";}
};


// ------------------------------------------------------------------------
// ObjRefTypeCode class
// ------------------------------------------------------------------------
class ObjRefTypeCode: public TypeCode
{
    private:
	static const CORBA::TypeInfo	_class_info;
    public:
	static const ObjRefTypeCode		TC_Object;

	ObjRefTypeCode(const char *name);
	ObjRefTypeCode(ValueString * interface_id); 
	ObjRefTypeCode(NCistream& strm);

        void veryPrettyPrint(NCostream& strm, int spaces=0) const;

	~ObjRefTypeCode() {}

	const char *name() const;
	void name(const char *interface_name);


	static const CORBA::TypeInfo *_desc();
	virtual const CORBA::TypeInfo *_type_info() const;
	virtual void *_safe_narrow(const CORBA::TypeInfo *)const;
	static ObjRefTypeCode *_narrow(const CORBA::Object *obj);
	const char *_interface_name() const { return "CORBA::ObjRefTypeCode";}
};

// ------------------------------------------------------------------------
// SequenceTypeCode class
// ------------------------------------------------------------------------
class SequenceTypeCode: public TypeCode
{
    private:
	static const CORBA::TypeInfo	_class_info;
    public:
        static const SequenceTypeCode	TC_Contexts;
        static const SequenceTypeCode	TC_Parameters;
        static const SequenceTypeCode	TC_Exceptions;
        static const SequenceTypeCode	TC_Attributes;
        static const SequenceTypeCode	TC_Operations;

        static const SequenceTypeCode   TC_DBKeySequence;
        static const SequenceTypeCode   TC_BaseInterfaces;

	SequenceTypeCode(ValueTypeCode* typeCode, ValueLong *maxlen); 
	SequenceTypeCode() : TypeCode(TypeCode::tk_sequence) {}

// Also need a SequenceTypeCode constructor suitable for 
// IDL compiler-generated TypeCode constants

	SequenceTypeCode(const TypeCode *tc1, long maxlen); 

        void veryPrettyPrint(NCostream& strm, int spaces=0) const;

	SequenceTypeCode(NCistream& strm);
	~SequenceTypeCode() { }

	void add_parameter(Value* parameter);

	const TypeCode& member_typecode() const;
	Long	max_length() const;
	Boolean is_primitive() const { return CORBA::FALSE; }

	static const CORBA::TypeInfo *_desc();
	virtual const CORBA::TypeInfo *_type_info() const;
	virtual void *_safe_narrow(const CORBA::TypeInfo *)const;
	static SequenceTypeCode *_narrow(const CORBA::Object *obj);
	const char *_interface_name() const { return "CORBA::SequenceTypeCode";}

};



// ------------------------------------------------------------------------
// ArrayTypeCode class
// ------------------------------------------------------------------------
class ArrayTypeCode: public TypeCode
{
    private:
	static const CORBA::TypeInfo	_class_info;
    public:

	ArrayTypeCode(ValueTypeCode* typeCode, ValueLong *length);
	ArrayTypeCode() : TypeCode(TypeCode::tk_array) {}

// Also need a ArrayTypeCode constructor suitable for 
// IDL compiler-generated TypeCode constants

	ArrayTypeCode(const TypeCode *tc1, long length); 
	ArrayTypeCode(NCistream& strm);

        void veryPrettyPrint(NCostream& strm, int spaces=0) const;

	~ArrayTypeCode() { }

	void add_parameter(Value* parameter);

	const TypeCode& member_typecode() const;
	Long	length() const;
	Boolean is_primitive() const { return CORBA::FALSE; }

	static const CORBA::TypeInfo *_desc();
	virtual const CORBA::TypeInfo *_type_info() const;
	virtual void *_safe_narrow(const CORBA::TypeInfo *)const;
	static ArrayTypeCode *_narrow(const CORBA::Object *obj);
	const char *_interface_name() const { return "CORBA::ArrayTypeCode";}
};


#endif
