#ifndef _value_h_
#define _value_h_
/****************************************************************************/
/*                                                                          */
/*  ORBeline (c)   is copyrighted by PostModern Computing Technologies, Inc */
/*                                                                          */
/*  Copyright 1993, 1994 by PostModern Computing Technologies, Inc.         */
/*                          1897 Landings Drive                             */
/*                          Mountain View, California, USA                  */
/*                          Telephone (415) 967-6169                        */
/*                          Facsimile (415) 967-6212                        */
/*                                                                          */
/*  This code is the sole property of PostModern Computing Technologies,    */
/*  Inc.  and is  protected  by  copyright under the laws of the United     */
/*  States.   This  program is confidential, proprietary, and  a  trade     */
/*  secret,  not  to  be disclosed  without written authorization  from     */
/*  PostModern Computing Technologies, Inc.  Value use,  duplication,  or   */
/*  disclosure  of  this  program  by other than  PostModern  Computing     */
/*  Technologies, Inc. and its assigned licensees is strictly forbidden     */
/*  by law.                                                                 */
/*                                                                          */
/*  All rights reserved.                                                    */
/*                                                                          */
/****************************************************************************/

class Value: public DSResource
{
	DECLARE_NCOBJECT_ABSTRACT_CLASS(Value);
    protected:
	Value() {}

    public:
        static Value* factory(TypeCode *);
        static Value* factory(const TypeCode& );

	virtual ~Value() {}


	virtual TypeCode::TCKind kind() const =0;
	virtual const TypeCode& typeCode() const=0;


	void operator=(const Value& val) {
		value(val);
	}
	virtual void value(const Value& val) = 0;
	virtual void  copyFrom(NCistream& strm) = 0;
	virtual Boolean is_primitive() const { return CORBA::TRUE; }

	virtual void  veryPrettyPrint(NCostream& strm, int spaces) const;

        operator short() const { return shortValue(); }
        operator long() const { return longValue(); }
        operator unsigned short() const { return ushortValue(); }
        operator unsigned long () const { return ulongValue(); }
        operator float() const { return floatValue(); }
        operator double() const { return doubleValue(); }
        operator char() const { return charValue(); }
#ifndef CRAY
        operator unsigned char() const { return ucharValue(); }
#endif
        operator const char *() const { return stringValue(); }
	operator Object *() const { return objrefValue(); }
        const Value& operator[](long i) const { return memberAt(i); }
        Value&  operator[] (long i) { return memberAt(i); }
	
	virtual void	add_member(Value *);

        void operator=(short val) { shortValue(val); }
        void operator=(long val) { longValue(val); }
        void operator=(unsigned short val) { ushortValue(val); }
        void operator=(unsigned long val) { ulongValue(val); }
        void operator=(float val) { floatValue(val); }
        void operator=(double val) {doubleValue(val); }
        void operator=(char val) { charValue(val); }
#ifndef CRAY
        void operator=(unsigned char val) { ucharValue(val); }
#endif
        void operator=(const char * val) { stringValue(val); }
	void operator=(CORBA::Object *val) { objrefValue(val); }



	// Functions implemented by derived classes
	virtual Value& member(const char *name);
	virtual const Value& member(const char *name) const;
	virtual ULong	member_count() const { return 0; }
	virtual Value& memberAt(Long index);
	virtual const Value& memberAt(Long index) const;

	virtual short  		shortValue() const;
        virtual long  		longValue() const;
	virtual unsigned short  ushortValue()const; 
        virtual unsigned long  	ulongValue()const;
        virtual float  		floatValue( ) const;
        virtual double  	doubleValue() const;
#ifndef CRAY
	virtual unsigned char  	ucharValue() const;
#endif
	virtual char  		charValue() const;
        virtual const char *  	stringValue() const;
        virtual const TypeCode&	typeCodeValue() const;
	virtual const Any&	anyValue() const;
	virtual Any&		anyValue();
	virtual Object		*objrefValue() const;

	virtual void  		shortValue(short);
        virtual void  		longValue(long);
	virtual void  		ushortValue(unsigned short);
        virtual void  		ulongValue(unsigned long);
        virtual void  		floatValue(float);
        virtual void  		doubleValue(double);
#ifndef CRAY
	virtual void  		ucharValue(unsigned char);
#endif
	virtual void  		charValue(char);
        virtual void  		stringValue(const char*);
        virtual void  		stringValue(const String&);
        virtual void  		typeCodeValue(const TypeCode&);
        virtual void  		anyValue(const Any&);
        virtual void  		anyValue(Any *value);
        virtual void  		objrefValue(Object *value);
};


class ValueShort: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueShort);
    private:
        short _value;
    public:
        ValueShort(short val=0) { _value = val; }
	~ValueShort() {}

	void copyFrom(NCistream& strm) { strm >> _value; }
	TypeCode::TCKind kind() const { return TypeCode::tk_short; }
	const TypeCode& typeCode() const { return TypeCode::TC_short; }

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
		shortValue(val.shortValue());
	}
	short shortValue() const { return _value; }
	void  shortValue(short val) { _value = val; }
};


class ValueLong: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueLong);
    private:
        long _value;
    public:
        ValueLong(long val=0) { _value = val; }
	~ValueLong() {}

	TypeCode::TCKind kind() const { return TypeCode::tk_long; }
	const TypeCode& typeCode() const { return TypeCode::TC_long; }
	void copyFrom(NCistream& strm) { strm >> _value; }

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
		longValue(val.longValue());
	}
	long longValue() const { return _value; }
	void  longValue(long val) { _value = val; }
};


class ValueUShort: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueUShort);
    private:
        unsigned short _value;
    public:
        ValueUShort(unsigned short val=0) { _value = val; }
	~ValueUShort() { }

	TypeCode::TCKind kind() const { return TypeCode::tk_ushort; }
	const TypeCode& typeCode() const { return TypeCode::TC_ushort; }
	void copyFrom(NCistream& strm) { strm >> _value; }

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
		ushortValue(val.ushortValue());
	}
	unsigned short ushortValue() const { return _value; }
	void  ushortValue(unsigned short val) { _value = val; }
};



class ValueULong: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueULong);
    private:
        unsigned long _value;
    public:
        ValueULong(unsigned long val=0) { _value = val; }
	~ValueULong() { }

	TypeCode::TCKind kind() const { return TypeCode::tk_ulong; }
	const TypeCode& typeCode() const { return TypeCode::TC_ulong; }
	void copyFrom(NCistream& strm) { strm >> _value; }

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
		ulongValue(val.ulongValue());
	}
	unsigned long ulongValue() const { return _value; }
	void  ulongValue(unsigned long val) { _value = val; }
};


class ValueFloat: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueFloat);
    private:
        float _value;
    public:
        ValueFloat(float val=0.0) { _value = val; }
	~ValueFloat() { }


	TypeCode::TCKind kind() const { return TypeCode::tk_float; }
	const TypeCode& typeCode() const { return TypeCode::TC_float; }
	void copyFrom(NCistream& strm) { strm >> _value; }

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
		floatValue(val.floatValue());
	}
	float floatValue() const { return _value; }
	void  floatValue(float val) { _value = val; }
};


class ValueDouble: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueDouble);
    private:
        double _value;
    public:
        ValueDouble(double val=0.0) { _value = val; }
	~ValueDouble() { }

	TypeCode::TCKind kind() const { return TypeCode::tk_double; }
	const TypeCode& typeCode() const { return TypeCode::TC_double; }
	void copyFrom(NCistream& strm) { strm >> _value; }

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
		doubleValue(val.doubleValue());
	}
	double doubleValue() const { return _value; }
	void  doubleValue(double val) { _value = val; }
};


class ValueOctet: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueOctet);
    private:
        unsigned char _value;
    public:
        ValueOctet(unsigned char val=0) { _value = val; }
	~ValueOctet() {}

	TypeCode::TCKind kind() const { return TypeCode::tk_octet; }
	const TypeCode& typeCode() const { return TypeCode::TC_octet; }
	void copyFrom(NCistream& strm) { strm >> _value; }

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

#if defined(CRAY)
	char charValue() const { return _value; }
	void charValue(char val) { _value = val; }
#else
	unsigned char ucharValue() const { return _value; }
	void  ucharValue(unsigned char val) { _value = val; }
#endif
	void value(const Value& val) {
#if defined(CRAY)
		charValue(val.charValue());
#else
		ucharValue(val.ucharValue());
#endif
	}
};


class ValueBoolean: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueBoolean);
    private:
        unsigned char _value;
    public:
        ValueBoolean(unsigned char val= 0) { _value = val; }
	~ValueBoolean() {}

	TypeCode::TCKind kind() const { return TypeCode::tk_boolean; }
	const TypeCode& typeCode() const { return TypeCode::TC_boolean; }
	void copyFrom(NCistream& strm) { strm >> _value; }

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
#if defined(CRAY)
		charValue(val.charValue());
#else
		ucharValue(val.ucharValue());
#endif
	}
#if defined(CRAY)
	char charValue() const { return _value; }
	void charValue(char val) { _value = val; }
#else
	unsigned char ucharValue() const { return _value; }
	void  ucharValue(unsigned char val) { _value = val; }
#endif
};


class ValueChar: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueChar);
    private:
        char _value;
    public:
        ValueChar(char val=0) { _value = val; }
	~ValueChar() {}

	TypeCode::TCKind kind() const { return TypeCode::tk_char; }
	const TypeCode& typeCode() const { return TypeCode::TC_char; }
	void copyFrom(NCistream& strm) { strm >> _value; }

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
		charValue(val.charValue());
	}
	char charValue() const { return _value; }
	void charValue(char val) { _value = val; }
};


class ValueString: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueString);
    private:
        String _value;
    public:
	ValueString() {}
        ValueString(const char *val) : _value(val) {}
        ValueString(const String& val) : _value(val) {}
	~ValueString() {}

	TypeCode::TCKind kind() const { return TypeCode::tk_string; }
	const TypeCode& typeCode() const { return StringTypeCode::TC_string; }
	void copyFrom(NCistream& strm) { _value.copyFrom(strm); }

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
		stringValue(val.stringValue());
	}
	const char  *stringValue() const { return (const char *)_value; }
	void  stringValue(const char * val) { _value = val; }
	void  stringValue(const String& val ) { _value = val; }
};

class ValueObjRef: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueObjRef);
    private:
        Object *_value;
    public:
	ValueObjRef();
        ValueObjRef(CORBA::Object *val);
	~ValueObjRef();

	TypeCode::TCKind kind() const { return TypeCode::tk_objref; }
	const TypeCode& typeCode() const { return ObjRefTypeCode::TC_Object; }
	void copyFrom(NCistream& strm);

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
		objrefValue(val.objrefValue());
	}
	Object *objrefValue() const { return _value; }
	void  objrefValue(Object* val);
};


class ValueTypeCode: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueTypeCode);
    private:
        TypeCode *_value;
    public:
	ValueTypeCode();
        ValueTypeCode(const TypeCode& value);
	ValueTypeCode(TypeCode *value);
	~ValueTypeCode()  { _value->_release(); }

	TypeCode::TCKind kind() const { return TypeCode::tk_TypeCode; }
	const TypeCode& typeCode() const { return TypeCode::TC_TypeCode; }
	void copyFrom(NCistream& strm);

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val) {
		typeCodeValue(val.typeCodeValue());
	}
        const TypeCode&  typeCodeValue() const;
        void  typeCodeValue(const TypeCode& val);
};


class ValueAny: public Value
{
	DECLARE_NCOBJECT_CLASS(ValueAny);
    private:
	Any	_value;
    public:
	ValueAny();
        ValueAny(const Any& value);
	ValueAny(Any *value);
	~ValueAny() {}

	TypeCode::TCKind kind() const { return TypeCode::tk_any; }
	const TypeCode& typeCode() const { return TypeCode::TC_any; }
	void copyFrom(NCistream& strm);

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void  value(const Value& val);

	void  anyValue(const Any& any);
	void  anyValue(Any *any);
	const Any& anyValue() const;
	Any&  anyValue();
};



class ValueConstructed: public Value
{
	DECLARE_NCOBJECT_ABSTRACT_CLASS(ValueConstructed);
    protected:
       	DSOrderedCltn 	_members;
	TypeCode	*_typeCode;

	ValueConstructed();
        ValueConstructed(const TypeCode& code);
	ValueConstructed(TypeCode *code);
	void		add_member(Value *);                              
    public:
	virtual void copyFrom(NCistream& strm);

        virtual ~ValueConstructed();
	virtual void value(const Value& val);

	TypeCode::TCKind kind() const { return _typeCode->kind(); }
	const TypeCode& typeCode() const { return *_typeCode; }

	virtual Boolean is_primitive() const { return CORBA::FALSE; }

        ULong member_count()  const { return (ULong)_members.size(); }

	virtual void  veryPrettyPrint(NCostream& strm, int spaces) const;

	const Value&	memberAt(Long index) const;
	Value&		memberAt(Long index);
};



class ValueStruct: public ValueConstructed
{
	DECLARE_NCOBJECT_CLASS(ValueStruct);
    private:
	void	_add_values();
    public:
        ValueStruct(const StructTypeCode& code);
        ValueStruct(StructTypeCode *code);
        ~ValueStruct() {}

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	Value&  member(const char *member_name);
	const Value&  member(const char *member_name) const;
	Long	member_index(const char *member_name) const;
	Boolean member_exists(const char *member_name) const { 
		return (member_index(member_name) >= 0);
	}
};

class ValueEnum: public ValueConstructed
{
	DECLARE_NCOBJECT_CLASS(ValueEnum);
    public:
        ValueEnum(const EnumTypeCode& code);
        ValueEnum(EnumTypeCode *code);
        ~ValueEnum() {}

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

        long  longValue() const;
        void  longValue(long value);
};

class ValueUnion: public ValueConstructed
{
	DECLARE_NCOBJECT_CLASS(ValueUnion);
    public:
	// A Union value should be constructed with a discriminator
	// value. If no discrim is passed, it is assumed that 
	// discriminator value is 0
        ValueUnion(const UnionTypeCode& code, 
			const Value *discrimValue=(Value *)NULL); 
        ValueUnion(UnionTypeCode *code, 
			const Value *discrimValue=(Value *)NULL);
	
        ~ValueUnion() {}
	void operator=(const ValueUnion& val) {
		value(val);
	}

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val);

	void	copyFrom(NCistream& strm);
};

class ValueArray: public ValueConstructed
{
	DECLARE_NCOBJECT_CLASS(ValueArray);
    private:
	void	_add_values();
    public:
        ValueArray(const ArrayTypeCode& code);
        ValueArray(ArrayTypeCode *code);
        ~ValueArray() {}

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void	write(NCostream& strm);
	void	copyFrom(NCistream& strm);
};

class ValueSequence: public ValueConstructed
{
	DECLARE_NCOBJECT_CLASS(ValueSequence);
    public:
        ValueSequence(const SequenceTypeCode& code);
        ValueSequence(SequenceTypeCode* code);
        ~ValueSequence() {}
	void operator=(const ValueSequence& seq) {
		value(seq);
	}

	void  veryPrettyPrint(NCostream& strm, int spaces) const;

	void value(const Value& val);

	void	copyFrom(NCistream& strm);
	void	write(NCostream& strm);
	void	add_member(Value *);                              
};

#endif
